@@ -1,12 +1,13 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright (c) 2019-2020 NXP
+# Copyright (c) 2019-2021 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Module implementing the Mboot communication protocol."""
 
 from .base import Interface
-from .usb import scan_usb, RawHid
-from .uart import scan_uart, Uart
+from .uart import Uart, scan_uart
+from .usb import RawHid, scan_usb
+from .usbsio import UsbSioI2C, UsbSioSPI, scan_usbsio
