@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020 NXP
+# Copyright 2020-2021 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -11,8 +11,14 @@ import os
 
 from spsdk import SPSDK_DATA_FOLDER
 
-PFR_DATA_FOLDER: str = os.path.join(SPSDK_DATA_FOLDER, 'pfr')
+PFR_DATA_FOLDER: str = os.path.join(SPSDK_DATA_FOLDER, "pfr")
 
-from .pfr import CMPA, CFPA
+from .pfr import CMPA, CFPA, PfrConfiguration
+from .exceptions import (
+    SPSDKPfrError,
+    SPSDKPfrConfigError,
+    SPSDKPfrConfigReadError,
+    SPSDKPfrRotkhIsNotPresent,
+)
 from .translator import Translator
 from .processor import Processor
