@@ -8,8 +8,11 @@
 """Translator is responsible for converting stringified keys into values."""
 
 import logging
+
 from .pfr import CFPA, CMPA, PfrConfiguration
 
+logger = logging.getLogger(__name__)
+
 
 class Translator:
     """Translates single strings (register/key names) into values."""
@@ -20,7 +23,6 @@ class Translator:
         :param cmpa: configuration data loaded from CMPA config file
         :param cfpa: configuration data loaded from CFPA config file
         """
-        self.logger = logging.getLogger("translator")
         self.cmpa_cfg = cmpa
         self.cmpa_obj = CMPA(device=cmpa.device, revision=cmpa.revision, user_config=cmpa)
         self.cfpa_cfg = cfpa
@@ -38,33 +40,33 @@ class Translator:
         :return: Register's (key's) value
         """
         area, value = key.split(".", maxsplit=1)
-        self.logger.debug(f"Area designator: {area}")
-        self.logger.debug(f"Register designator: {value}")
+        logger.debug(f"Area designator: {area}")
+        logger.debug(f"Register designator: {value}")
         return self.handlers[area](value)
 
     def _cmpa_translate(self, key: str) -> int:
         """Handler for CMPA data."""
-        self.logger.debug(f"Extracting value from {key}")
+        logger.debug(f"Extracting value from {key}")
         splitted = key.split(".", maxsplit=1)
         register = self.cmpa_obj.registers.find_reg(splitted[0])
         if len(splitted) == 2:
             value = register.find_bitfield(splitted[1]).get_value()
         else:
             value = register.get_int_value()
-        self.logger.debug(f"Extracted value {value:x}")
+        logger.debug(f"Extracted value {value:x}")
 
         return value
 
     def _cfpa_translate(self, key: str) -> int:
         """Handler for CFPA data."""
-        self.logger.debug(f"Extracting value from {key}")
+        logger.debug(f"Extracting value from {key}")
         splitted = key.split(".", maxsplit=1)
         register = self.cfpa_obj.registers.find_reg(splitted[0])
         if len(splitted) == 2:
             value = register.find_bitfield(splitted[1]).get_value()
         else:
             value = register.get_int_value()
-        self.logger.debug(f"Extracted value {value:x}")
+        logger.debug(f"Extracted value {value:x}")
 
         return value
 
